/**
 * Created by Ludo Pulles on 2017-12-26.
 */

/*******************************************************************************
 * Gomoku is the object which manages the gomoku visualisation.
 * This part makes sure that the board transitions can be made.
 * When the board is changed, it sends the board to all its controllers.
 ******************************************************************************/

function Gomoku(playerNames, moves) {
	this.playerNames = playerNames;
	this.chara = 'a'.charCodeAt(0);
	this.charA = 'A'.charCodeAt(0);
	this.size = 16;
	var self = this;

	this.moves = [ "start" ];
	for (var i in moves) {
		if (moves[i] == 'Zz') this.moves.push("swap");
		else this.moves.push(self.size * (moves[i].charCodeAt(0) - self.charA) + (moves[i].charCodeAt(1) - self.chara));
	}

	this.isRunning = false;

	this.board = document.getElementById('game-container');
	this.moveTable = document.getElementById('move-table').children[0];

	var showLegends = true;

	// create the board
	var table = document.createElement('table');
	var thead = document.createElement('thead');
	var thr = document.createElement('tr');

	if (showLegends) {
		thr.appendChild(document.createElement('th'));

		for (var col = 0; col < this.size; col++) {
			var th = document.createElement('th');
			th.innerText = String.fromCharCode(this.chara + col);
			th.id = "col" + (col + 1);
			thr.appendChild(th);
		}
		thead.appendChild(thr);
		table.appendChild(thead);
	}

	this.cells = new Array(this.size * this.size);
	var tbody = document.createElement('tbody');
	for (var row = 0; row < this.size; row++) {
		var tr = document.createElement('tr');

		if (showLegends) {
			var rowName = String.fromCharCode(this.charA + row);
			var rowElem = document.createElement('td');
			rowElem.innerText = rowName;
			rowElem.className = "frow";
			rowElem.id = "row" + (row + 1);
			tr.appendChild(rowElem);
		}
		for (var col = 0; col < this.size; col++) {
			var stone = document.createElement('div');
			var cur = document.createElement('td');
			cur.className = "game-cell";

			cur.appendChild(stone);
			tr.appendChild(cur);
			this.cells[row * this.size + col] = stone;
		}
		tbody.appendChild(tr);
	}

	var X = document.createElement('tr');
	tbody.appendChild(X);
	var btnwrapper = document.createElement('td');
	btnwrapper.setAttribute('colspan', this.size + 1);
	btnwrapper.setAttribute('style', 'text-align:center;');
	btnwrapper.appendChild(document.getElementsByClassName('button-bar')[0]);
	X.appendChild(btnwrapper);
	table.appendChild(tbody);

	this.board.appendChild(table);

	this.curMove = 0;

	this.registerEvents();
	this.buildMoveTable(moves);
	this.updateSelectedRow();
}

// returns whether the move has been processed
Gomoku.prototype.setMove = function(move) {
	if (move < 0 || move >= this.moves.length) return false;
	if (move == this.curMove) return true;

	while (this.curMove > move) {
		// change a move back.
		// undo this.curMove
		var cur = this.moves[this.curMove--];
		if (cur == "swap") {
			// swap
			this.cells[this.moves[1]].className = 'p1';
			this.cells[this.moves[2]].className = 'p2';
			this.cells[this.moves[3]].className = 'p1';
		} else this.cells[cur].className = '';
	}
	while (this.curMove < move) {
		// process this.curMove
		var cur = this.moves[++this.curMove];
		var owner = this.curMove % 2 == 0 ? 'p2' : 'p1';
		if (cur == "swap") {
			this.cells[this.moves[1]].className = 'p2';
			this.cells[this.moves[2]].className = 'p1';
			this.cells[this.moves[3]].className = 'p2';
		} else this.cells[cur].className = owner;
	}

	this.updateSelectedRow();
	return true;
};

Gomoku.prototype.setRunning = function(running) {
	if (this.isRunning == running) return;

	this.isRunning = running;
	this.playPause.innerText = running ? 'Pause' : 'Play';

	var self = this, run = function() {
		if (!self.isRunning) return;

		if (self.setMove(self.curMove + 1)) {
			setTimeout(run, 500);
		} else {
			// last move has been reached.
			self.setRunning(false);
		}
	}
	if (this.isRunning) run();
}

Gomoku.prototype.prevMove = function() { this.setRunning(false); this.setMove(this.curMove - 1); };
Gomoku.prototype.nextMove = function() { this.setRunning(false); this.setMove(this.curMove + 1); };

Gomoku.prototype.registerEvents = function() {
	this.playPause = document.getElementById('play-pause');

	var self = this;
	document.getElementById('first').onclick = function() { self.setRunning(false); self.setMove(0); };
	document.getElementById('last').onclick = function() { self.setRunning(false); self.setMove(self.moves.length - 1); };
	document.getElementById('prev').onclick = function() { self.prevMove(); };
	document.getElementById('next').onclick = function() { self.nextMove(); };
	this.playPause.onclick = function() { self.setRunning(!self.isRunning); };
};


Gomoku.prototype.buildMoveTable = function(moves) {
	var table = document.createElement('table'), self = this;
	var theader = document.createElement('tr');
	theader.appendChild(document.createElement('th'));
	for (var i = 0; i < 2; i++) {
		var th = document.createElement('th');

		// cut-off on more than 20 characters in the name
		var playerName = i == 0 ? "black" : "white";
		if (this.playerNames) {
			var name = this.playerNames[i];
			if (name.length > 20) name = name.slice(0, 17) + "...";

			playerName = name + " as " + playerName;
		}

		th.innerText = playerName;
		theader.appendChild(th);
	}

	var thead = document.createElement('thead');
	thead.appendChild(theader);
	table.appendChild(thead);

	var tbody = document.createElement('tbody');

	for (var i = 0; i <= moves.length; i++) {
		var tr = document.createElement('tr');
		var moveNr = document.createElement('th');
		moveNr.innerText = i + '.';
		if (i % 2) tr.setAttribute('class', 'zebra');
		tr.appendChild(moveNr);

		var curp = i < 3 ? 0 : ~i & 1;
		for (var j = 0; j < 2; j++) {
			var td = document.createElement('td');
			if (j == curp) td.innerText = !i ? "Start" : moves[i - 1];
			tr.appendChild(td);
		}

		tr.onclick = (function(move) {
			return function() { self.setMove(move); }
		})(i);

		tbody.appendChild(tr);
	}

	this.moveTableBody = tbody;
	this.moveTable.onkeydown = function(event) {
		if (event.keyCode == 38 || event.keyCode == 37) {
			self.prevMove();
		} else if (event.keyCode == 40 || event.keyCode == 39) {
			self.nextMove();
		} else {
			return;
		}
		event.preventDefault();
		event.stopImmediatePropagation();
	};

	table.appendChild(tbody);
	this.moveTable.appendChild(table);
};

Gomoku.prototype.updateSelectedRow = function() {
	var lastCurMove = document.getElementById('cur-move');
	var tbl = this.moveTable;

	// reset the id.
	if (lastCurMove) lastCurMove.id = '';

	if (this.curMove >= 0) {
		var curRow = this.moveTableBody.children[this.curMove];
		curRow.id = 'cur-move';

		var topCoord = curRow.offsetTop;
		var bottomCoord = curRow.offsetTop + curRow.offsetHeight - tbl.offsetHeight;
		if (tbl.scrollTop < bottomCoord) {
			tbl.scrollTop = bottomCoord;
		} else if (tbl.scrollTop > topCoord) {
			tbl.scrollTop = topCoord;
		}
	} else {
		tbl.scrollTop = 0;
	}
};
